"""
Test Item connections.
"""

from gaphor.tests import TestCase
from zope import component
from gaphor import UML
from gaphor.diagram import items
from gaphor.diagram.interfaces import IConnect


class ConnectorTestCase(TestCase):
    pass


# fixme: relationship test moved from multi dependency test
